@@ -437,12 +437,21 @@ static std::string getMangledTypeStr(Type *Ty) {
       Result += "vararg";
     // Ensure nested function types are distinguishable.
     Result += "f";
-  } else if (isa<VectorType>(Ty))
+  } else if (isa<VectorType>(Ty)) {
     Result += "v" +
               utostr(VCINTR::VectorType::getNumElements(cast<VectorType>(Ty))) +
               getMangledTypeStr(cast<VectorType>(Ty)->getElementType());
-  else if (Ty)
+#if VC_INTR_LLVM_VERSION_MAJOR >= 16
+  } else if (auto *TargetTy = dyn_cast<TargetExtType>(Ty)) {
+    Result += "t_" + TargetTy->getName().str();
+    for (auto *PTy : TargetTy->type_params())
+      Result += "_" + getMangledTypeStr(PTy);
+    for (auto I : TargetTy->int_params())
+      Result += "_" + llvm::utostr(I);
+#endif // VC_INTR_LLVM_VERSION_MAJOR >= 16
+  } else if (Ty) {
     Result += EVT::getEVT(Ty).getEVTString();
+  }
 
   return Result;
 }
@@ -521,7 +530,7 @@ static ArrayRef<const char *> findTargetSubtable(StringRef Name) {
   // We've either found the target or just fall back to the generic set, which
   // is always first.
   const auto &TI = It != Targets.end() && It->Name == Target ? *It : Targets[0];
-  return makeArrayRef(&GenXIntrinsicNameTable[1] + TI.Offset, TI.Count);
+  return ArrayRef<const char *>(&GenXIntrinsicNameTable[1] + TI.Offset, TI.Count);
 }
 
 GenXIntrinsic::ID GenXIntrinsic::getGenXIntrinsicID(const Function *F) {
