@@ -318,7 +318,8 @@ static bool isArgConvIntrinsic(const Value *V) {
 static Value *getOriginalValue(Argument &Arg) {
   if (Arg.hasOneUse()) {
     User *U = Arg.user_back();
-    if (isArgConvIntrinsic(U))
+    if (isArgConvIntrinsic(U) || isa<BitCastInst>(U) ||
+        isa<AddrSpaceCastInst>(U) || isa<IntToPtrInst>(U))
       return U;
   }
 
@@ -468,7 +469,7 @@ transformKernelSignature(Function &F, const std::vector<SPIRVArgDesc> &Descs) {
 // Rewrite function if it has SPIRV types as parameters.
 // Function
 //  define spir_kernel @foo(%opencl.image2d_rw_t addrspace(1)* %im) {
-//    %conv = call @llvm.genx.address.convert(%im)
+//    %conv = ptrtoint %opencl.image2d_rw_t addrspace(1)* %im to i32
 //   ...
 //  }
 // will be changed to
@@ -486,8 +487,24 @@ static void rewriteKernelArguments(Function &F) {
     // All arguments are in old style.
     return;
 
-  // No uses. Fast composite is not converted in writer part for now.
-  assert(F.use_empty() && "FC is not supported yet");
+  // At the moment there are only two cases when kernel function with converted
+  // parameters can have users:
+  // 1. Kernel is called from another function via fast composite
+  //    For such kernels we just don't rewrite arguments on SPIRV write, so
+  //    there should not be presented on read
+  // 2. Kernel is referenced in @llvm.global.annotations
+  //    We have to replace the original function with the new one
+  if (!F.use_empty()) {
+    assert(F.hasOneUse());
+    auto *Bitcast = F.user_back();
+    assert(Bitcast->hasOneUse());
+    auto *Struct = Bitcast->user_back();
+    assert(Struct->hasOneUse());
+    auto *Array = Struct->user_back();
+    assert(Array->hasOneUse());
+    auto *GV = dyn_cast<GlobalVariable>(Array->user_back());
+    assert(GV && GV->getName() == "llvm.global.annotations");
+  }
 
   Function *NewF = transformKernelSignature(F, ArgDescs);
   F.getParent()->getFunctionList().insert(F.getIterator(), NewF);
@@ -521,6 +538,8 @@ static void rewriteKernelArguments(Function &F) {
     }
   }
 
+  F.mutateType(NewF->getType());
+  F.replaceAllUsesWith(NewF);
   F.eraseFromParent();
 }
 
@@ -546,7 +565,7 @@ bool GenXSPIRVReaderAdaptorImpl::run(Module &M) {
   if (KernelMDs)
     return false;
 
-  for (auto &&GV : M.getGlobalList()) {
+  for (auto &&GV : M.globals()) {
     if (!GV.hasAttribute(VCModuleMD::VCGlobalVariable))
       continue;
     if (GV.hasAttribute(VCModuleMD::VCVolatile))
